package twmoead.twmoead;

import jmetal.core.Solution;
import jmetal.core.SolutionSet;
import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;
import twmoead.utils.StrictRanking;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;

/**
 * Created by Martin on 28.01.2016.
 */
public class CustomFuncUserObjective implements UserObjective {

    @Override
    public double value(double[][] objectives) {

        SolutionSet sols = new SolutionSet(objectives.length);
        for (int i = 0; i < objectives.length; i++) {
            Solution s = new Solution(2);
            s.setObjective(0, objectives[i][0]);
            s.setObjective(1, objectives[i][1]);
            sols.add(s);
        }

        StrictRanking sr = new StrictRanking(sols);

        double[][] sorted = extractObjectives(sr.getSubfront(0));
        Arrays.sort(sorted, new dObjectiveComparator(0));

        DescriptiveStatistics ds = new DescriptiveStatistics();
        for (int i = 0; i < sorted.length - 1; i++) {
            double dist = Math.pow(sorted[i][0] - sorted[i+1][0], 2) + Math.pow(sorted[i][1] - sorted[i+1][1], 2);
            dist = Math.sqrt(dist);
            ds.addValue(dist);
        }

        double avg = ds.getSum()/objectives.length;
        double maxD = ds.getMax();
        double minD = ds.getMin();
        double std = ds.getStandardDeviation();

        return avg - 4*std;
    }


    double[][] extractObjectives(SolutionSet pop) {
        double[][] objectives = new double[pop.size()][];
        for (int i = 0; i < objectives.length; i++) {
            objectives[i] = new double[pop.get(0).getNumberOfObjectives()];
        }

        for (int i = 0; i < objectives.length; i++) {
            Solution s = pop.get(i);
            for (int j = 0; j < objectives[0].length; j++) {
                objectives[i][j] = s.getObjective(j);
            }
        }

        return objectives;
    }


    class dObjectiveComparator implements Comparator<double []> {

        int idx;

        public dObjectiveComparator(int idx) {
            this.idx = idx;
        }

        @Override
        public int compare(double[] o1, double[] o2) {
            return Double.compare(o1[idx], o2[idx]);
        }

        @Override
        public boolean equals(Object obj) {
            return (obj instanceof dObjectiveComparator) && ((dObjectiveComparator) obj).idx == this.idx;
        }

    }

    double[][] removeDominated(double[][] objectives) {

        double[][] sorted = objectives.clone();
        Arrays.sort(sorted, new dObjectiveComparator(0));

        ArrayList<double[]> nonDom = new ArrayList<>();
        for (int i = 0; i < sorted.length - 1; i++) {
            if (sorted[i][1] > sorted[i+1][1])
                nonDom.add(sorted[i]);
        }

        double[][] res = new double[nonDom.size()][2];
        return nonDom.toArray(res);
    }
}
